function A = ComputeSystemMatrix( Coords, B1_mat, delta_t, IntSamples, k_mat, B0_vec )

    % this method computes the system matrix to obtain the mutlichannel RF 
    % pulse
    
    %error handling
    if size( Coords, 2 ) ~= 3
        error('Coords matrix must have three columns!') 
    end
    
    if size( Coords, 1 ) ~= size( B1_mat, 1 ) || size( Coords, 1 ) ~= size( B0_vec, 1 )
    	error('Coords, B1 and B0 matrices must have same number of rows (Coords = %i, B1 = %i, B0 = %i)!',size( Coords, 1 ),size( B1_mat, 1 ),size( B0_vec, 1 )) 
    end

    Gamma = 42.577*1e+6;
    %for better readability, set variables
    NumPixel    = size( Coords,1 );
    NumChannel  = size( B1_mat, 2 );
    Numksamples = size( k_mat, 1 );
    A           = ones( NumPixel, NumChannel * Numksamples ) + 1j * ones( NumPixel, NumChannel * Numksamples );
        
    %add one sample at the end
    k_mat           = [ k_mat; k_mat(end,:) + diff( k_mat(end-1:end,:) ) ];
    
    tmp_mat = nan( NumChannel, NumPixel );
    
    %precompute B1 factors
    for chn = 1 : NumChannel
        tmp_mat(chn,:) = 2 * pi * Gamma * B1_mat(:,chn).' * delta_t;
    end

    for kidx = 1 : Numksamples
        
        tmp2    = exp( -1j * 2 * pi * (Numksamples - kidx + 0.5) * delta_t * B0_vec.' ) .* ((-exp(2 * pi * k_mat(kidx,:) * Coords' * 1j) + exp(2 * pi * k_mat(kidx+1,:) * Coords' * 1j)) * 1j)./(2 * pi * (B0_vec' * -delta_t + (k_mat(kidx,:) - k_mat(kidx+1,:)) * Coords'));
        NaNidx  =  find( isnan(tmp2) | isinf(tmp2) );    
        if ~isempty(NaNidx)
            tmp2(NaNidx) = sum( exp( -1j * 2 * pi * ([  linspace( k_mat(kidx,1), k_mat(kidx+1,1), IntSamples + 1 ).', ...
                                                        linspace( k_mat(kidx,2), k_mat(kidx+1,2), IntSamples + 1 ).', ...
                                                    	linspace( k_mat(kidx,3), k_mat(kidx+1,3), IntSamples + 1 ).' ] * Coords(NaNidx,:)' + repmat( ( (Numksamples - kidx + 0.5) * delta_t )' * B0_vec(NaNidx)', IntSamples + 1, 1 ) ) ) )  ./ (IntSamples + 1);
        end
        
        for chn = 1 : NumChannel
            A(:,(chn-1) * Numksamples + kidx) = 1j * tmp2 .* tmp_mat(chn,:);
        end
    end

    %some final error handling
    if any(isnan(A(:))) || any(isinf(A(:)))
        save('SystemMatrixErrorData.mat','Coords', 'B1_mat', 'delta_t', 'IntSamples', 'k_mat', 'B0_vec')
        error('System matrix contains invalid entries!') 
    end
    
end
    