
close all;
clear

options.verbosity       = 2;  
% 0: only command outputs
% 1: plot final gradients and RF pulses
% 2: plot maps, magnetization, bloch sim etc. before optimization

%  S Y S T E M  P A R A M E T E R S
%-----------------------------------
options.Gmax            = 40.0;     % [mT/m]
options.Smax            = 150.0;   	% [T/m/s]
options.Vmax            = 150.0;    % [V]

%  F I E L D S   M A P S
%--------------------------
load('maps_head.mat');

options.b1maps          = b1maps;
options.b0map           = b0map;
options.roi             = roi;
options.xpos_vec        = xpos_vec;
options.ypos_vec        = ypos_vec;
options.zpos_vec        = zpos_vec;

options.undersam_x      = 2;   
options.undersam_y      = 2; 
options.undersam_z      = 1;

%  O P T I O N S
%------------------
% for additional options, have a loog at CheckOptions.m
options.blur_target     = 0.003; %[m]
options.objfun_Tmax     = 0.004; %[s]
options.objfun_beta     = 2;     %[-]
options.pulse_maxtime   = 0.008; %[s]
options.optim_rfsolver  = 1;     %[-]

options.use_gpu         = false;  % set to true, if a GPU is available
options.optim_debugmode = false;  % useful for adjusting initial trajectory;
                                  % in this mode no optimization will be
                                  % performed and no data will be stored

%  M A G N E T I Z A T I O N   T A R G E T
%-------------------------------------------

%use some predefined target pattern, or set an arbitrary target pattern in 
%the options.targetflip_mat matrix
TargetType = 1;
% 1: Cube
% 2: Brain
% 3: Userdefined

switch TargetType
    case 1
        %generate cube target magnetization pattern
        target_cx   = 0;        % target x centering in m
        target_cy   = 0;        % target y centering in m
        target_cz   = 0;        % target z centering in m 
        target_lx   = 0.04;     % cube full x dim, here 4x4x4 cm
        target_ly   = 0.04;     % cube full y dim
        target_lz   = 0.04;     % cube full z dim
        target_flip	= 10.0;     % target flip angle
        t_fun                   = @(x,y,z) double( abs(x - target_cx) <= target_lx/2 & abs(y - target_cy) <= target_ly/2 & abs(z - target_cz) <= target_lz/2 );
        [xmat,ymat,zmat]        = ndgrid( options.xpos_vec, options.ypos_vec, options.zpos_vec );
        options.targetflip_mat 	= t_fun(xmat,ymat,zmat) .* target_flip;
    case 2
        %generate brain target magnetization pattern
        load('files\brain.mat')
        target_flip             = 10;
        options.targetflip_mat  = brain .* target_flip; 
    case 3
        %place your code here...
end
        
%  K - S P A C E   T R A J E C T O R Y
%----------------------------------------

TrajectoryType = 1;
% 1: Shells
% 2: StackOfSpirals
% 3: Cross

switch TrajectoryType
    case 1
        options.trajectory_fun  = 'TrajectoryControlPoints_Shells';
        options.phi_init        = [   65.0000   65.0000   65.0000    3.0000    5.0000    0.0000    1.0000 ].';
        options.phi_bounds      = [   10.0000   10.0000   10.0000    2.0000    1.0000   -8.0000    0.5000 ; ...
                                     100.0000  100.0000  100.0000    7.5000   10.0000   +8.0000    2.0000 ].';
    case 2
        options.trajectory_fun  = 'TrajectoryControlPoints_StackOfSpirals';
        options.phi_init        = [ 65.0000   65.0000   65.0000   25.0000    2.0000    1.0000    1.0000 ].';
        options.phi_bounds      = [ 10.0000   10.0000   10.0000    1.0000    1.0000    0.0000    0.5000 ; ...
                                   100.0000  100.0000  100.0000   60.0000   10.0000    3.0000    2.0000 ].';
    case 3
        options.trajectory_fun  = 'TrajectoryControlPoints_Cross';
        options.phi_init        = [ 40.0000    0.4500   40.0000    0.4500   40.0000    0.4500 ].';
        options.phi_bounds      = [ 10.0000    0.0500   10.0000    0.0500   10.0000    0.0500 ; ...
                                   100.0000    1.0000  100.0000    1.0000  100.0000    1.0000 ].';
end
            
                        
%  R U N  S H A P E  O P T I M I Z A T I O N
%---------------------------------------------

[ PulseData, AuxData ] = PerformShapeOptimization( options ); 

% this function returns two structs
%
% PulseData with the following fields
% -----------------------------------
%         RF: complex RF coefficients, one row per channel
%         Gx: gradient waveforms in x
%         Gy: gradient waveforms in y
%         Gz: gradient waveforms in z
%          t: time samples
%
% If options.RF_delay is unequal to zero, the delayed RF and gradient
% waveforms will also be included
%
% RF_delayed: delayed complex RF coefficients, one row per channel
% Gx_delayed: delayed gradient waveforms in x
% Gy_delayed: delayed gradient waveforms in y
% Gz_delayed: delayed gradient waveforms in z
%  t_delayed: delayed time samples
%
% AuxData with the following fields
% ---------------------------------
%       blochmagn: Bloch simulation of final RF pulse
%      objfundata: Objective function data of final RF pulse
%               T: Trajectory struct
%              RF: RF pulse in a single column
%     forwardmagn: Small tip angle magnetization (results from 
%                  multiplication of the system matrix times the RF vector)
%         options: Options, as they were used during the shape optimization


