function PhiAbsolute = GetAbsoluteParameters( PhiRelative, PhiBounds )

    % Convert Relative ([0,...,1]) to Absolute shape parameters.

    %error handling
    if size( PhiRelative,1 ) ~= size( PhiBounds,1 )
       error('PhiAbsolute and PhiBounds must have same length!') 
    end
    if size( PhiBounds,2 ) ~= 2
       error('PhiBounds must have two columns!') 
    end
    if any( PhiRelative < 0.0 | PhiRelative > 1.0 )
        abserr = max( [abs( min( PhiRelative, 0.0 ) ); abs( max( PhiRelative - 1.0, 0.0 ) )] );
        fprintf('PhiRelative violates the given bounds, %i/%i entries are affected (abs(relerr) = %.2e)\n',sum(PhiRelative < 0.0 | PhiRelative > 1.0), numel(PhiRelative),abserr)
        PhiRelative( PhiRelative < 0.0 ) = 0.0;
        PhiRelative( PhiRelative > 1.0 ) = 1.0;
    end
    
    %convert to relative shape parameters
    PhiAbsolute = PhiRelative .* (PhiBounds(:,2) - PhiBounds(:,1)) + PhiBounds(:,1);

end

