function PhiRelative = GetRelativeParameters( PhiAbsolute, PhiBounds )

    % Convert Absolute to Relative ([0,...,1]) shape parameters.

    %error handling
    if size( PhiAbsolute,1 ) ~= size( PhiBounds,1 )
       error('PhiAbsolute and PhiBounds must have same length!') 
    end
    if size( PhiBounds,2 ) ~= 2
       error('PhiBounds must have two columns!') 
    end
    if any( PhiAbsolute < PhiBounds(:,1) | PhiAbsolute > PhiBounds(:,2) )
        phirel_temp = (PhiAbsolute - PhiBounds(:,1))./(PhiBounds(:,2) - PhiBounds(:,1));
        abserr      = max( [abs( min( phirel_temp, 0.0 ) ); abs( max( phirel_temp - 1.0, 0.0 ) )] );
        fprintf('PhiAbsolute violates the given bounds, %i/%i entries are affected (abs(relerr) = %.2e)\n',sum(phirel_temp < 0.0 | phirel_temp > 1.0), numel(phirel_temp),abserr)
        PhiAbsolute( PhiAbsolute < PhiBounds(:,1) ) = PhiBounds( PhiAbsolute < PhiBounds(:,1),1 );
        PhiAbsolute( PhiAbsolute > PhiBounds(:,2) ) = PhiBounds( PhiAbsolute > PhiBounds(:,2),2 );
    end
    
    %convert to relative shape parameters
    PhiRelative = (PhiAbsolute - PhiBounds(:,1))./(PhiBounds(:,2) - PhiBounds(:,1));

end

