function RF_sparse = GetSparseRF( RF_full, T )

    % Convert full RF waveforms (one sample per time sample) to sparse RF
    % waveforms (samples only where RF is supposed to be played during the
    % excitation)

    %input has to be full RF of length T.Pulse.RFfullLength
    if numel( RF_full ) ~= T.Pulse.RFfullLength
        if numel( RF_full ) / T.Pulse.NumChannel == T.Pulse.RFfullLength_perChannel - 1
            %RF_full is missing some zeros at the end
            RF_full	= [reshape( RF_full(:), numel( RF_full ) / T.Pulse.NumChannel, T.Pulse.NumChannel ); ...
                           zeros( 1,  T.Pulse.NumChannel) ];
         	RF_full = RF_full(:);
        elseif numel( RF_full ) / T.Pulse.NumChannel == T.Pulse.RFfullLength_perChannel + 1
            %RF has zero elements at the end, we may just remove these
            %elements
            FirstZero   = size(RF_full, 1) / T.Pulse.NumChannel;
            RF_full( FirstZero : FirstZero  : end ) = [];
        else
            error('Input RF has to have length of number of Hardware samples + 1 (numel(RF) = %i, numel(Hardware.t) = %i)',numel(RF_full),numel(T.Pulse.t)) 
        end
    end

    %allocate complex memory
    RF_sparse = nan( T.Pulse.MatSize, 1 ) + 1j * nan( T.Pulse.MatSize, 1 );
    
    %set nonzero entries
    NonZero_idx = T.Pulse.RFfull_idx( T.Pulse.RFfull_idx ~= 0 );
    RF_sparse( unique( NonZero_idx ) ) = RF_full( T.Pulse.RFfull_NonZeros == 1 );
    
    RF_sparse( isnan( RF_full ) ) = 0.0;