
%gradient hardware constraints
Gmax        = 25.0;     %[mT/m]
Smax        = 150.0;   	%[T/m/s]

verbosity   = 2;
% 0: no output
% 1: objective function options and final results
% 2: objective function options, final results, iteration output

%tuning of optimization
TolFun 	= 1e-8; %Default 1e-6
TolX  	= 1e-8; %Default 1e-6
TolCon  = 1e-8; %Default 1e-6

UseBoundaryConstraints = 1;
%set to 1 to force, that first and last constrol point satisfy the gradient
%boundary constraint G(0) = (Gx_init, Gy_init, Gz_init)
%and                 G(T) = (Gx_end,  Gy_end,  Gz_end)              
Gx_start    = 0.0;
Gy_start    = 0.0;
Gx_end      = 0.0;
Gy_end      = 0.0;

% Optimal Gradient Design Solver
%-------------------------------
SolverID_idx    = 4;
ValidSolvers = {'InterPoint_Jacobian',...   % 1: fast and good
            	'InterPoint_Hessian',...    % 2: fast and good
                'MinConF_SPG',...           % 4: slow, very fast trajectories
                'ModifiedShor'};            % 9: slow, fastest trajectories
SolverID = ValidSolvers{ SolverID_idx };


%Some options of the objective function
%--------------------------------------
Options.OnlyContinuousRanges    = 0;
Options.GradSmooth_On           = 1;
Options.GradSmooth_Lambda       = 1e-3; %[ms]
Options.GradSmooth_MaxDistFac   = 2;
Options.verbosity              	= verbosity;
Options.UseMex                  = 0;
Options.SolverID                = SolverID;


% Use some predefined trajectory
%-------------------------------
CPSet = 2;
%  1: Test Pattern
%  2: Spiral without vector constraints
%  3: Spiral with vector constraints
%  4: EPI without vector constraints
%  5: EPI with vector constraints

switch CPSet
    case 1
        load('ConPoints_2D_TestPattern.mat')
    case 2
        load('ConPoints_2D_Spiral_unconstrained.mat')
    case 3
        load('ConPoints_2D_Spiral_constrained.mat')     
    case 4
        load('ConPoints_2D_EPI_unconstrained.mat')   
    case 5
        load('ConPoints_2D_EPI_constrained.mat')        
end

%set boundary constraints
if UseBoundaryConstraints
    R_mat(1,:)      = [Gx_start, Gy_start, 0];
    R_mat(end,:)    = [Gx_end, Gy_end, 0];
end

% Compute Gradient Waveforms
%---------------------------
G0  = []; %start with zero gradient in constrained optimization
T   = PerformOptimization2D( kx_vec, ky_vec, R_mat, Gmax, Smax, G0, Options );

% plot trajectory
PlotTrajectory( T );

% Get hardware-suitable sampling
%-------------------------------
dT  = 10e-6;
T   = CalculateHardwareTrajectory( T, dT );

% plot gradient waveforms using hardware sampling
%(analytic gradients can be plotted by changing 'h' to 'gs')
PlotGradients( T, 'h' );



