function [Gx, Gy, Gz] = GetGradientVectors( G_var, AuxData )

    %based on the constraint matrix R_mat in AuxData, back out the full
    %gradient vectors from the vector representing the variables in the
    %optimization problem
    
    %error handling
    if numel( G_var ) ~= sum( AuxData.R( :,end ) )
        error('Input vector and constraint matrix are inconsistent (NV_input = %i, NV_constr = %i)',numel( G_var ), sum( AuxData.R( :,end )) )
    end

    NCP = size( AuxData.R, 1 );
    Dim = size( AuxData.R, 2 ) - 1;
    
    switch Dim
        case 2
            % 2D - Trajectory Design Problem
            %-------------------------------
            Gx  = nan( 1,NCP );
            Gy  = nan( 1,NCP );
            Gz  = zeros( 1,NCP );
            G_var_idx   = 1;
            
            for k = 1 : NCP
                if AuxData.R(k,3) == 2
                    %vector had full number of DoF
                    Gx( k )  	= G_var( G_var_idx );
                    Gy( k )     = G_var( G_var_idx+1 );
                    G_var_idx  	= G_var_idx + 2;
                    
                elseif AuxData.R(k,3) == 1
                    %vector had a single DoF
                    Gx( k )  	= G_var( G_var_idx ) * AuxData.R(k,1);
                    Gy( k )     = G_var( G_var_idx ) * AuxData.R(k,2);
                    G_var_idx  	= G_var_idx + 1;
                    
                elseif AuxData.R(k,3) == 0
                    %vector had fixed gradients
                    Gx( k )  	= AuxData.R(k,1);
                    Gy( k )     = AuxData.R(k,2);
                    
                else
                    error('Slope constraint array seems to be invalid!')
                end  
            end
            
            
            
        case 3
            % 3D - Trajectory Design Problem
            %-------------------------------
            Gx  = nan( 1,NCP );
            Gy  = nan( 1,NCP );
            Gz  = nan( 1,NCP );
            G_var_idx   = 1;
            
            for k = 1 : NCP
                if AuxData.R(k,4) == 3
                    %vector had full number of DoF
                    Gx( k )  	= G_var( G_var_idx );
                    Gy( k )     = G_var( G_var_idx+1 );
                    Gz( k )     = G_var( G_var_idx+2 );
                    G_var_idx  	= G_var_idx + 3;
                    
                elseif AuxData.R(k,4) == 1
                    %vector had a single DoF
                    Gx( k )  	= G_var( G_var_idx ) * AuxData.R(k,1);
                    Gy( k )     = G_var( G_var_idx ) * AuxData.R(k,2);
                    Gz( k )     = G_var( G_var_idx ) * AuxData.R(k,3);
                    G_var_idx  	= G_var_idx + 1;
                    
                elseif AuxData.R(k,4) == 0
                    %vector had fixed gradients
                    Gx( k )  	= AuxData.R(k,1);
                    Gy( k )     = AuxData.R(k,2);
                    Gz( k )     = AuxData.R(k,2);
                    
                else
                    error('Slope constraint array seems to be invalid!')
                end  
            end  
    end
    
end

