function f = EvalObjective2D_fval( x, AuxData )    

    %#codegen

    %{ 
    Method:
    -------
    Evaluates the objective function (equivalent to the pulse duration)
    based on the vector x to be optimized (x constains the variable 
    gradient magnitudes to be optimized) and the AuxData struct. AuxData
    contains information about the constraint on the control points,
    several options such as the smoothing behaviour, etc. In the
    method the gradients are solved within two subsequent control points
    based on the current gradient values at the control points. In addition
    the Jacobian and the Hessian are stored and formated accordingly to the
    constraints
    
    Input:
    ------
    
    x       - vector of gradient magnitudes
    AuxData - struct (cf. SetUpAuxData2D/3D)

    Output:
    -------
    
    f   - objective function value at x
    
    %}
    
    NCP = size(AuxData.R,1);
    f   = 0.0;

    %******************************************************************
    %           C O M P U T E   O B J E C T I V E   O N L Y 
    %******************************************************************
    %fprintf('Evaluating (%i): Objective\n',nargout)

    %back out full gradient vector
    G_full  = AuxData.D' * x;
    
    for k = 1 : NCP-1

        %evaluate objective function using the previously defined gradient
        %vectors, depending on the number of variables to be returned, the 
        % associated method is used
        fk = FastTimeOptGrad_2D_deriv0( G_full( 2*k-1 ), G_full( 2*k+1 ), G_full( 2*k ), G_full( 2*k+2 ), ...
                                        AuxData.dkx(k), AuxData.dky(k), ...
                                        AuxData.Gmax, AuxData.Smax, AuxData.OptsVector );
        f	= f + fk;
    end
 
end
    
    
    

