function [tlim, gradIn, gradOut, gradIn_In, gradIn_Out, gradOut_In, gradOut_Out] = GetMinimumTime3D( lowBoundsA, uppBoundsA, lowBoundsB, uppBoundsB, lowBoundsC, uppBoundsC, OptsVector )

    %#codegen
    %this method is used to determine the smallest tlim contained in both
    %ranges A and B, given by n time ranges in lowBoundsA and uppBoundsA
    %and m time ranges in lowBoundsB and uppBoundsB. The method should find
    %a tlim in any case, since each dimension contains a range with upper
    %bound being infinity
  
    %GradLevel
    if      size( lowBoundsA, 2 ) == 1
        GradLevel = 1;
    elseif  size( lowBoundsA, 2 ) == 3
        GradLevel = 2;
    else    %size( lowBoundsA, 2 ) == 7
        GradLevel = 3; 
    end

    %initialize everything
    gradIn      = [];
    gradOut     = [];
    gradIn_In   = []; 
    gradIn_Out  = []; 
    gradOut_In  = []; 
    gradOut_Out = [];

    %preferences
    UseWeightedGradient = OptsVector(2) ;
    Lambda              = OptsVector(3); %[ms]
    MaxDist             = OptsVector(4) * Lambda;
    
 	%predefintion
    InitIdx     = find( max([lowBoundsA(end,1),lowBoundsB(end,1),lowBoundsC(end,1)]) == max([lowBoundsA(end,1),lowBoundsB(end,1),lowBoundsC(end,1)]), 1, 'first');
    switch InitIdx(1)
        case 1
            tlim        = lowBoundsA(end,1);
        case 2
            tlim        = lowBoundsB(end,1);
        otherwise
            tlim        = lowBoundsC(end,1);
    end
    
    DistanceA   = tlim - lowBoundsA(end,1);
    DistanceB   = tlim - lowBoundsB(end,1);
    DistanceC   = tlim - lowBoundsC(end,1);
    
    %set initial gradients
    if GradLevel == 2
        [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'init', UseWeightedGradient );
    elseif GradLevel == 3
        [gradIn, gradOut, ...
         gradIn_In, gradIn_Out, ...
         gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'init', UseWeightedGradient );
    end

    while(1)
        
        %get largest of lowest bounds in A and B
        CurLim      = double( find(max([lowBoundsA(1),lowBoundsB(1),lowBoundsC(1)]) == [lowBoundsA(1),lowBoundsB(1),lowBoundsC(1)], 1) );
        
        switch CurLim(1)
            case 1
                %check, if lowest Bound of A is in B
                % YES: return
                %  NO: remove first range of B
                Bool = [max( lowBoundsA(1,1) <= uppBoundsB(:,1) & lowBoundsA(1,1) >= lowBoundsB(:,1) ) == 1, ...
                        max( lowBoundsA(1,1) <= uppBoundsC(:,1) & lowBoundsA(1,1) >= lowBoundsC(:,1) ) == 1 ];
                if all(Bool)   
                    tlim        = lowBoundsA(1,1);
                    DistanceA   = tlim - lowBoundsA(1,1);
                    DistanceB   = tlim - lowBoundsB(1,1);
                    DistanceC   = tlim - lowBoundsC(1,1);

                    %set gradients
                    if GradLevel == 2
                        [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'inter', UseWeightedGradient );
                    elseif GradLevel == 3
                        [gradIn, gradOut, ...
                         gradIn_In, gradIn_Out, ...
                         gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'inter', UseWeightedGradient );
                    end
                    return;
                else
                    %only delete the intervals which are not solvable
                    if ~Bool(1)
                        lowBoundsB(1,:)   = [];
                        uppBoundsB(1,:)   = [];
                    end
                    if ~Bool(2)
                        lowBoundsC(1,:)   = [];
                        uppBoundsC(1,:)   = [];
                    end
                    
                end
                
        	case 2
                %check, if lowest Bound of B is in A & C
                % YES: return
                %  NO: remove first range of A & C
                Bool = [max( lowBoundsB(1,1) <= uppBoundsA(:,1) & lowBoundsB(1,1) >= lowBoundsA(:,1) ) == 1,...
                        max( lowBoundsB(1,1) <= uppBoundsC(:,1) & lowBoundsB(1,1) >= lowBoundsC(:,1) ) == 1  ];
                if all(Bool)   
                    tlim        = lowBoundsB(1,1);
                    DistanceA   = tlim - lowBoundsA(1,1);
                    DistanceB   = tlim - lowBoundsB(1,1);
                    DistanceC   = tlim - lowBoundsC(1,1);

                    %set gradients
                    if GradLevel == 2
                        [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'inter', UseWeightedGradient );
                    elseif GradLevel == 3
                        [gradIn, gradOut, ...
                         gradIn_In, gradIn_Out, ...
                         gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'inter', UseWeightedGradient );
                    end
                    return;
                else
                    if ~Bool(1)
                        lowBoundsA(1,:)   = [];
                        uppBoundsA(1,:)   = [];
                    end
                    if ~Bool(2)
                        lowBoundsC(1,:)   = [];
                        uppBoundsC(1,:)   = [];
                    end
                end
                
            case 3
                %check, if lowest Bound of C is in A & B
                % YES: return
                %  NO: remove first range of A & B
                Bool = [max( lowBoundsC(1,1) <= uppBoundsA(:,1) & lowBoundsC(1,1) >= lowBoundsA(:,1) ) == 1, ...
                        max( lowBoundsC(1,1) <= uppBoundsB(:,1) & lowBoundsC(1,1) >= lowBoundsB(:,1) ) == 1];
                if all(Bool)     
                    tlim        = lowBoundsC(1,1);
                    DistanceA   = tlim - lowBoundsA(1,1);
                    DistanceB   = tlim - lowBoundsB(1,1);
                    DistanceC   = tlim - lowBoundsC(1,1);

                    %set gradients
                    if GradLevel == 2
                        [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'inter', UseWeightedGradient );
                    elseif GradLevel == 3
                        [gradIn, gradOut, ...
                         gradIn_In, gradIn_Out, ...
                         gradOut_In, gradOut_Out ] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, 'inter', UseWeightedGradient );
                    end
                    return;
                else
                    if ~Bool(1)
                        lowBoundsA(1,:)   = [];
                        uppBoundsA(1,:)   = [];
                    end
                    if ~Bool(2)
                        lowBoundsB(1,:)   = [];
                        uppBoundsB(1,:)   = [];
                    end
                    
                end
        end

        if isempty(lowBoundsA) || isempty(uppBoundsA) || ...
           isempty(lowBoundsB) || isempty(uppBoundsB) || ...
           isempty(lowBoundsC) || isempty(uppBoundsC)
            break;
        end

    end 
    
end

%Compute weighted Jacobian
function [gradIn, gradOut] = SetWeightedGradients_Jac( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, state, UseWeightedGradient )

    %#codegen
    %entries at Idx are kept as original, other entries are weighted

    switch state
        case 'init'
            BndAIdx = size( lowBoundsA, 1 );
            BndBIdx = size( lowBoundsB, 1 );
            BndCIdx = size( lowBoundsC, 1 );
        case 'inter'
            BndAIdx = 1;
            BndBIdx = 1;
            BndCIdx = 1;
    end
    
    %get effective weights
    if (UseWeightedGradient && DistanceA < MaxDist) || (DistanceA == 0)
        DistanceA_eff = DistanceA;
    else
        DistanceA_eff = +Inf;
    end
    if (UseWeightedGradient && DistanceB < MaxDist) || (DistanceB == 0)
        DistanceB_eff = DistanceB;
 	else
        DistanceB_eff = +Inf;
    end
    if (UseWeightedGradient && DistanceC < MaxDist) || (DistanceC == 0)
        DistanceC_eff = DistanceC;
  	else
        DistanceC_eff = +Inf;
    end
    
    %set gradients
    gradIn  = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,2), ...
                exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,2), ...
                exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,2) ];
  	gradOut = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,3), ...
                exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,3), ...
                exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,3) ];
    
end

%Compute weighted Jacobian
function [gradIn, gradOut, gradIn_In, gradIn_Out, gradOut_In, gradOut_Out] = SetWeightedGradients_Hess( lowBoundsA, lowBoundsB, lowBoundsC, DistanceA, DistanceB, DistanceC, MaxDist, Lambda, state, UseWeightedGradient )

    %#codegen
    % entries at Idx are kept as original, other entries are weighted

    switch state
        case 'init'
            BndAIdx = size( lowBoundsA, 1 );
            BndBIdx = size( lowBoundsB, 1 );
            BndCIdx = size( lowBoundsC, 1 );
        case 'inter'
            BndAIdx = 1;
            BndBIdx = 1;
            BndCIdx = 1;
    end
    
    %get effective weights
    if (UseWeightedGradient && DistanceA < MaxDist) || (DistanceA == 0)
        DistanceA_eff = DistanceA;
    else
        DistanceA_eff = +Inf;
    end
    if (UseWeightedGradient && DistanceB < MaxDist) || (DistanceB == 0)
        DistanceB_eff = DistanceB;
 	else
        DistanceB_eff = +Inf;
    end
    if (UseWeightedGradient && DistanceC < MaxDist) || (DistanceC == 0)
        DistanceC_eff = DistanceC;
  	else
        DistanceC_eff = +Inf;
    end
    
    %set gradients
    gradIn      = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,2), ...
                    exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,2), ...
                    exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,2) ];
  	gradOut     = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,3), ...
                    exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,3), ...
                    exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,3) ];
  	gradIn_In   = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,4), ...
                    exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,4), ...
                    exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,4) ];
  	gradIn_Out  = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,5), ...
                    exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,5), ...
                    exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,5) ];
  	gradOut_In  = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,6), ...
                    exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,6), ...
                    exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,6) ];
  	gradOut_Out = [ exp(-DistanceA_eff/Lambda) * lowBoundsA(BndAIdx,7), ...
                    exp(-DistanceB_eff/Lambda) * lowBoundsB(BndBIdx,7), ...
                    exp(-DistanceC_eff/Lambda) * lowBoundsC(BndCIdx,7) ];
end


